@@ -20,6 +20,8 @@ import org.apache.spark.SparkThrowable;
 import org.apache.spark.SparkThrowableHelper;
 import org.apache.spark.annotation.Private;
 
+import java.util.Map;
+
 /**
  * This exception is thrown when a task can not acquire memory from the Memory manager.
  * Instead of throwing {@link OutOfMemoryError}, which kills the executor,
@@ -28,7 +30,7 @@ import org.apache.spark.annotation.Private;
 @Private
 public final class SparkOutOfMemoryError extends OutOfMemoryError implements SparkThrowable {
     String errorClass;
-    String[] messageParameters;
+    Map<String, String> messageParameters;
 
     public SparkOutOfMemoryError(String s) {
         super(s);
@@ -38,12 +40,18 @@ public final class SparkOutOfMemoryError extends OutOfMemoryError implements Spa
         super(e.getMessage());
     }
 
-    public SparkOutOfMemoryError(String errorClass, String[] messageParameters) {
-        super(SparkThrowableHelper.getMessage(errorClass, messageParameters, ""));
+    public SparkOutOfMemoryError(String errorClass, Map<String, String> messageParameters) {
+        super(SparkThrowableHelper.getMessage(errorClass, messageParameters));
         this.errorClass = errorClass;
         this.messageParameters = messageParameters;
     }
 
+    @Override
+    public Map<String, String> getMessageParameters() {
+        return messageParameters;
+    }
+
+    @Override
     public String getErrorClass() {
         return errorClass;
     }
