@@ -40,6 +40,7 @@ import org.apache.spark.util.{Clock, SystemClock, Utils}
  *      stage, but still many failures over the entire application
  *  * "flaky" executors -- they don't fail every task, but are still faulty enough to merit
  *      excluding
+ *  * missing shuffle files -- may trigger fetch failures on healthy executors.
  *
  * See the design doc on SPARK-8425 for a more in-depth discussion. Note SPARK-32037 renamed
  * the feature.
@@ -61,9 +62,11 @@ private[scheduler] class HealthTracker (
   HealthTracker.validateExcludeOnFailureConfs(conf)
   private val MAX_FAILURES_PER_EXEC = conf.get(config.MAX_FAILURES_PER_EXEC)
   private val MAX_FAILED_EXEC_PER_NODE = conf.get(config.MAX_FAILED_EXEC_PER_NODE)
-  val EXCLUDE_ON_FAILURE_TIMEOUT_MILLIS = HealthTracker.getExludeOnFailureTimeout(conf)
+  val EXCLUDE_ON_FAILURE_TIMEOUT_MILLIS = HealthTracker.getExcludeOnFailureTimeout(conf)
   private val EXCLUDE_FETCH_FAILURE_ENABLED =
     conf.get(config.EXCLUDE_ON_FAILURE_FETCH_FAILURE_ENABLED)
+  private val EXCLUDE_ON_FAILURE_DECOMMISSION_ENABLED =
+    conf.get(config.EXCLUDE_ON_FAILURE_DECOMMISSION_ENABLED)
 
   /**
    * A map from executorId to information on task failures. Tracks the time of each task failure,
@@ -90,7 +93,7 @@ private[scheduler] class HealthTracker (
    * remove from this when executors are removed from spark, so we can track when we get multiple
    * successive excluded executors on one node.  Nonetheless, it will not grow too large because
    * there cannot be many excluded executors on one node, before we stop requesting more
-   * executors on that node, and we clean up the list of exluded executors once an executor has
+   * executors on that node, and we clean up the list of excluded executors once an executor has
    * been excluded for EXCLUDE_ON_FAILURE_TIMEOUT_MILLIS.
    */
   val nodeToExcludedExecs = new HashMap[String, HashSet[String]]()
@@ -107,7 +110,7 @@ private[scheduler] class HealthTracker (
       // Apply the timeout to excluded nodes and executors
       val execsToInclude = executorIdToExcludedStatus.filter(_._2.expiryTime < now).keys
       if (execsToInclude.nonEmpty) {
-        // Include any executors that have been exluded longer than the excludeOnFailure timeout.
+        // Include any executors that have been excluded longer than the excludeOnFailure timeout.
         logInfo(s"Removing executors $execsToInclude from exclude list because the " +
           s"the executors have reached the timed out")
         execsToInclude.foreach { exec =>
@@ -154,11 +157,21 @@ private[scheduler] class HealthTracker (
   }
 
   private def killExecutor(exec: String, msg: String): Unit = {
+    val fullMsg = if (EXCLUDE_ON_FAILURE_DECOMMISSION_ENABLED) {
+      s"${msg} (actually decommissioning)"
+    } else {
+      msg
+    }
     allocationClient match {
       case Some(a) =>
-        logInfo(msg)
-        a.killExecutors(Seq(exec), adjustTargetNumExecutors = false, countFailures = false,
-          force = true)
+        logInfo(fullMsg)
+        if (EXCLUDE_ON_FAILURE_DECOMMISSION_ENABLED) {
+          a.decommissionExecutor(exec, ExecutorDecommissionInfo(fullMsg),
+            adjustTargetNumExecutors = false)
+        } else {
+          a.killExecutors(Seq(exec), adjustTargetNumExecutors = false, countFailures = false,
+            force = true)
+        }
       case None =>
         logInfo(s"Not attempting to kill excluded executor id $exec " +
           s"since allocation client is not defined.")
@@ -182,10 +195,18 @@ private[scheduler] class HealthTracker (
     if (conf.get(config.EXCLUDE_ON_FAILURE_KILL_ENABLED)) {
       allocationClient match {
         case Some(a) =>
-          logInfo(s"Killing all executors on excluded host $node " +
-            s"since ${config.EXCLUDE_ON_FAILURE_KILL_ENABLED.key} is set.")
-          if (a.killExecutorsOnHost(node) == false) {
-            logError(s"Killing executors on node $node failed.")
+          if (EXCLUDE_ON_FAILURE_DECOMMISSION_ENABLED) {
+            logInfo(s"Decommissioning all executors on excluded host $node " +
+              s"since ${config.EXCLUDE_ON_FAILURE_KILL_ENABLED.key} is set.")
+            if (!a.decommissionExecutorsOnHost(node)) {
+              logError(s"Decommissioning executors on $node failed.")
+            }
+          } else {
+            logInfo(s"Killing all executors on excluded host $node " +
+              s"since ${config.EXCLUDE_ON_FAILURE_KILL_ENABLED.key} is set.")
+            if (!a.killExecutorsOnHost(node)) {
+              logError(s"Killing executors on node $node failed.")
+            }
           }
         case None =>
           logWarning(s"Not attempting to kill executors on excluded host $node " +
@@ -361,7 +382,7 @@ private[scheduler] class HealthTracker (
     /**
      * Apply the timeout to individual tasks.  This is to prevent one-off failures that are very
      * spread out in time (and likely have nothing to do with problems on the executor) from
-     * triggering exlusion.  However, note that we do *not* remove executors and nodes from
+     * triggering exclusion.  However, note that we do *not* remove executors and nodes from
      * being excluded as we expire individual task failures -- each have their own timeout.  E.g.,
      * suppose:
      *  * timeout = 10, maxFailuresPerExec = 2
@@ -426,7 +447,7 @@ private[spark] object HealthTracker extends Logging {
     }
   }
 
-  def getExludeOnFailureTimeout(conf: SparkConf): Long = {
+  def getExcludeOnFailureTimeout(conf: SparkConf): Long = {
     conf.get(config.EXCLUDE_ON_FAILURE_TIMEOUT_CONF).getOrElse {
       conf.get(config.EXCLUDE_ON_FAILURE_LEGACY_TIMEOUT_CONF).getOrElse {
         Utils.timeStringAsMs(DEFAULT_TIMEOUT)
@@ -463,7 +484,7 @@ private[spark] object HealthTracker extends Logging {
       }
     }
 
-    val timeout = getExludeOnFailureTimeout(conf)
+    val timeout = getExcludeOnFailureTimeout(conf)
     if (timeout <= 0) {
       // first, figure out where the timeout came from, to include the right conf in the message.
       conf.get(config.EXCLUDE_ON_FAILURE_TIMEOUT_CONF) match {
