@@ -18,7 +18,7 @@
 package org.apache.spark.scheduler.cluster
 
 import java.util.Locale
-import java.util.concurrent.Semaphore
+import java.util.concurrent.{Semaphore, TimeUnit}
 import java.util.concurrent.atomic.AtomicBoolean
 
 import scala.concurrent.Future
@@ -26,13 +26,16 @@ import scala.concurrent.Future
 import org.apache.spark.{SparkConf, SparkContext}
 import org.apache.spark.deploy.{ApplicationDescription, Command}
 import org.apache.spark.deploy.client.{StandaloneAppClient, StandaloneAppClientListener}
+import org.apache.spark.executor.ExecutorExitCode
 import org.apache.spark.internal.{config, Logging}
+import org.apache.spark.internal.config.EXECUTOR_REMOVE_DELAY
 import org.apache.spark.internal.config.Tests.IS_TESTING
 import org.apache.spark.launcher.{LauncherBackend, SparkAppHandle}
-import org.apache.spark.resource.ResourceUtils
-import org.apache.spark.rpc.RpcEndpointAddress
+import org.apache.spark.resource.ResourceProfile
+import org.apache.spark.rpc.{RpcAddress, RpcEndpointAddress}
 import org.apache.spark.scheduler._
-import org.apache.spark.util.Utils
+import org.apache.spark.scheduler.cluster.CoarseGrainedClusterMessages.RemoveExecutor
+import org.apache.spark.util.{ThreadUtils, Utils}
 
 /**
  * A [[SchedulerBackend]] implementation for Spark's standalone cluster manager.
@@ -45,7 +48,7 @@ private[spark] class StandaloneSchedulerBackend(
   with StandaloneAppClientListener
   with Logging {
 
-  private var client: StandaloneAppClient = null
+  private[spark] var client: StandaloneAppClient = null
   private val stopping = new AtomicBoolean(false)
   private val launcherBackend = new LauncherBackend() {
     override protected def conf: SparkConf = sc.conf
@@ -59,6 +62,11 @@ private[spark] class StandaloneSchedulerBackend(
 
   private val maxCores = conf.get(config.CORES_MAX)
   private val totalExpectedCores = maxCores.getOrElse(0)
+  private val defaultProf = sc.resourceProfileManager.defaultResourceProfile
+
+  private val executorDelayRemoveThread =
+    ThreadUtils.newDaemonSingleThreadScheduledExecutor("driver-executor-delay-remove-thread")
+  private val _executorRemoveDelay = conf.get(EXECUTOR_REMOVE_DELAY)
 
   override def start(): Unit = {
     super.start()
@@ -81,7 +89,8 @@ private[spark] class StandaloneSchedulerBackend(
       "--hostname", "{{HOSTNAME}}",
       "--cores", "{{CORES}}",
       "--app-id", "{{APP_ID}}",
-      "--worker-url", "{{WORKER_URL}}")
+      "--worker-url", "{{WORKER_URL}}",
+      "--resourceProfileId", "{{RESOURCE_PROFILE_ID}}")
     val extraJavaOpts = sc.conf.get(config.EXECUTOR_JAVA_OPTIONS)
       .map(Utils.splitCommandString).getOrElse(Seq.empty)
     val classPathEntries = sc.conf.get(config.EXECUTOR_CLASS_PATH)
@@ -110,15 +119,18 @@ private[spark] class StandaloneSchedulerBackend(
     // ExecutorAllocationManager will send the real initial limit to the Master later.
     val initialExecutorLimit =
       if (Utils.isDynamicAllocationEnabled(conf)) {
+        if (coresPerExecutor.isEmpty) {
+          logWarning("Dynamic allocation enabled without spark.executor.cores explicitly " +
+            "set, you may get more executors allocated than expected. It's recommended to " +
+            "set spark.executor.cores explicitly. Please check SPARK-30299 for more details.")
+        }
+
         Some(0)
       } else {
         None
       }
-    val executorResourceReqs = ResourceUtils.parseResourceRequirements(conf,
-      config.SPARK_EXECUTOR_PREFIX)
-    val appDesc = ApplicationDescription(sc.appName, maxCores, sc.executorMemory, command,
-      webUrl, sc.eventLogDir, sc.eventLogCodec, coresPerExecutor, initialExecutorLimit,
-      resourceReqsPerExecutor = executorResourceReqs)
+    val appDesc = ApplicationDescription(sc.appName, maxCores, command,
+      webUrl, defaultProfile = defaultProf, sc.eventLogDir, sc.eventLogCodec, initialExecutorLimit)
     client = new StandaloneAppClient(sc.env.rpcEnv, masters, appDesc, this, conf)
     client.start()
     launcherBackend.setState(SparkAppHandle.State.SUBMITTED)
@@ -165,15 +177,34 @@ private[spark] class StandaloneSchedulerBackend(
   }
 
   override def executorRemoved(
-      fullId: String, message: String, exitStatus: Option[Int], workerLost: Boolean): Unit = {
+      fullId: String,
+      message: String,
+      exitStatus: Option[Int],
+      workerHost: Option[String]): Unit = {
     val reason: ExecutorLossReason = exitStatus match {
+      case Some(ExecutorExitCode.HEARTBEAT_FAILURE) =>
+        ExecutorExited(ExecutorExitCode.HEARTBEAT_FAILURE, exitCausedByApp = false, message)
+      case Some(ExecutorExitCode.DISK_STORE_FAILED_TO_CREATE_DIR) =>
+        ExecutorExited(ExecutorExitCode.DISK_STORE_FAILED_TO_CREATE_DIR,
+          exitCausedByApp = false, message)
       case Some(code) => ExecutorExited(code, exitCausedByApp = true, message)
-      case None => SlaveLost(message, workerLost = workerLost)
+      case None => ExecutorProcessLost(message, workerHost, causedByApp = workerHost.isEmpty)
     }
     logInfo("Executor %s removed: %s".format(fullId, message))
     removeExecutor(fullId.split("/")(1), reason)
   }
 
+  override def executorDecommissioned(fullId: String,
+      decommissionInfo: ExecutorDecommissionInfo): Unit = {
+    logInfo(s"Asked to decommission executor $fullId")
+    val execId = fullId.split("/")(1)
+    decommissionExecutors(
+      Array((execId, decommissionInfo)),
+      adjustTargetNumExecutors = false,
+      triggeredByExecutor = false)
+    logInfo("Executor %s decommissioned: %s".format(fullId, decommissionInfo))
+  }
+
   override def workerRemoved(workerId: String, host: String, message: String): Unit = {
     logInfo("Worker %s removed: %s".format(workerId, message))
     removeWorker(workerId, host, message)
@@ -195,9 +226,12 @@ private[spark] class StandaloneSchedulerBackend(
    *
    * @return whether the request is acknowledged.
    */
-  protected override def doRequestTotalExecutors(requestedTotal: Int): Future[Boolean] = {
+  protected override def doRequestTotalExecutors(
+      resourceProfileToTotalExecs: Map[ResourceProfile, Int]): Future[Boolean] = {
+    // resources profiles not supported
     Option(client) match {
-      case Some(c) => c.requestTotalExecutors(requestedTotal)
+      case Some(c) =>
+        c.requestTotalExecutors(resourceProfileToTotalExecs)
       case None =>
         logWarning("Attempted to request executors before driver fully initialized.")
         Future.successful(false)
@@ -235,6 +269,7 @@ private[spark] class StandaloneSchedulerBackend(
   private def stop(finalState: SparkAppHandle.State): Unit = {
     if (stopping.compareAndSet(false, true)) {
       try {
+        executorDelayRemoveThread.shutdownNow()
         super.stop()
         if (client != null) {
           client.stop()
@@ -250,4 +285,66 @@ private[spark] class StandaloneSchedulerBackend(
     }
   }
 
+  override def createDriverEndpoint(): DriverEndpoint = {
+    new StandaloneDriverEndpoint()
+  }
+
+  private class StandaloneDriverEndpoint extends DriverEndpoint {
+    // [SC-104659]: There are two paths to detect executor loss.
+    // (1) (fast path) `onDisconnected`: Executor -> Driver
+    //     When Executor closes its JVM, the socket (Netty's channel) will be closed. The
+    //     function onDisconnected will be triggered when driver knows the channel is closed.
+    //
+    // (2) (slow path) ExecutorRunner -> Worker -> Master -> Driver
+    //     When executor exits with ExecutorExitCode, the exit code will be passed from
+    //     ExecutorRunner to Driver. (Check [SC-104335] PR for details)
+    //
+    // Both path will call the function `removeExecutor` to remove the lost executor. The main
+    // difference between these two paths is ExecutorExitCode. To elaborate, the ExecutorLossReason
+    // of slow path has the information of ExecutorExitCode, but fast path does not have. Hence,
+    // slow path can determine the category of the executor loss with more information.
+    //
+    // Typically, fast path will be triggered prior to slow path. That is, when driver receives the
+    // ExecutorExitCode from slow path, the lost executor has already been removed from
+    // executorDataMap by fast path. Hence, we delay to send RemoveExecutor(executorId, lossReason)
+    // by _executorRemoveDelay milliseconds when the function onDisconnected is triggered, and hope
+    // to receive ExecutorExitCode from slow path during the delay.
+    override def onDisconnected(remoteAddress: RpcAddress): Unit = {
+      addressToExecutorId.get(remoteAddress).foreach { executorId =>
+        // [SC-104659]:
+        // When driver detects executor loss by fast path (`onDisconnected`), we need to notify
+        // task scheduler to avoid assigning new tasks on this lost executor and wait slow path
+        // for `_executorRemoveDelay` seconds. To prevent assigning tasks to the lost executor,
+        // we added the executor to `executorsPendingLossReason`. Hence, the executor will be
+        // filtered out from `activeExecutors` in the function `getWorkerOffers`.
+        executorsPendingLossReason += executorId
+        val lossReason = ExecutorProcessLost("Remote RPC client disassociated. Likely due to " +
+          "containers exceeding thresholds, or network issues. Check driver logs for WARN " +
+          "messages.")
+        val removeExecutorTask = new Runnable() {
+          override def run(): Unit = Utils.tryLogNonFatalError {
+            // If the executor is not removed by slow path, fast path will send a `RemoveExecutor`
+            // message to the scheduler backend.
+            //
+            // [Note]: Here may have race condition because `executorsPendingLossReason` will be
+            //         operated in the following 3 cases for standalone scheduler.
+            //
+            //  1. `removeExecutor`: executorsPendingLossReason -= executorId (remove)
+            //  2. `onDisconnected`: executorsPendingLossReason += executorId (add)
+            //  3. `executorDelayRemoveThread`: executorsPendingLossReason.contains(executorId)
+            //
+            // Case 1 & case 3 may have race condition. Case 2 & case 3 may also have. However,
+            // race condition is okay because `removeExecutor` will check whether the executor is
+            // existing or not. If the executor has been removed, the extra `RemoveExecutor`
+            // message will have no effectiveness.
+            if (executorsPendingLossReason.contains(executorId)) {
+              driverEndpoint.send(RemoveExecutor(executorId, lossReason))
+            }
+          }
+        }
+        executorDelayRemoveThread.schedule(removeExecutorTask,
+          _executorRemoveDelay, TimeUnit.MILLISECONDS)
+      }
+    }
+  }
 }
