@@ -20,7 +20,9 @@ package org.apache.spark.storage
 import java.util.UUID
 
 import org.apache.spark.SparkException
-import org.apache.spark.annotation.DeveloperApi
+import org.apache.spark.annotation.{DeveloperApi, Since}
+import org.apache.spark.errors.SparkCoreErrors
+import org.apache.spark.network.shuffle.RemoteBlockPushResolver
 
 /**
  * :: DeveloperApi ::
@@ -42,6 +44,7 @@ sealed abstract class BlockId {
     (isInstanceOf[ShuffleBlockId] || isInstanceOf[ShuffleBlockBatchId] ||
      isInstanceOf[ShuffleDataBlockId] || isInstanceOf[ShuffleIndexBlockId])
   }
+  def isShuffleChunk: Boolean = isInstanceOf[ShuffleBlockChunkId]
   def isBroadcast: Boolean = isInstanceOf[BroadcastBlockId]
 
   override def toString: String = name
@@ -71,6 +74,17 @@ case class ShuffleBlockBatchId(
   }
 }
 
+@Since("3.2.0")
+@DeveloperApi
+case class ShuffleBlockChunkId(
+    shuffleId: Int,
+    shuffleMergeId: Int,
+    reduceId: Int,
+    chunkId: Int) extends BlockId {
+  override def name: String =
+    "shuffleChunk_" + shuffleId  + "_" + shuffleMergeId + "_" + reduceId + "_" + chunkId
+}
+
 @DeveloperApi
 case class ShuffleDataBlockId(shuffleId: Int, mapId: Long, reduceId: Int) extends BlockId {
   override def name: String = "shuffle_" + shuffleId + "_" + mapId + "_" + reduceId + ".data"
@@ -81,6 +95,66 @@ case class ShuffleIndexBlockId(shuffleId: Int, mapId: Long, reduceId: Int) exten
   override def name: String = "shuffle_" + shuffleId + "_" + mapId + "_" + reduceId + ".index"
 }
 
+@Since("3.2.0")
+@DeveloperApi
+case class ShuffleChecksumBlockId(shuffleId: Int, mapId: Long, reduceId: Int) extends BlockId {
+  override def name: String = "shuffle_" + shuffleId + "_" + mapId + "_" + reduceId + ".checksum"
+}
+
+@Since("3.2.0")
+@DeveloperApi
+case class ShufflePushBlockId(
+    shuffleId: Int,
+    shuffleMergeId: Int,
+    mapIndex: Int,
+    reduceId: Int) extends BlockId {
+  override def name: String = "shufflePush_" + shuffleId + "_" +
+    shuffleMergeId + "_" + mapIndex + "_" + reduceId + ""
+}
+
+@Since("3.2.0")
+@DeveloperApi
+case class ShuffleMergedBlockId(
+    shuffleId: Int,
+    shuffleMergeId: Int,
+    reduceId: Int) extends BlockId {
+  override def name: String = "shuffleMerged_" + shuffleId + "_" +
+    shuffleMergeId + "_" + reduceId
+}
+
+@Since("3.2.0")
+@DeveloperApi
+case class ShuffleMergedDataBlockId(
+    appId: String,
+    shuffleId: Int,
+    shuffleMergeId: Int,
+    reduceId: Int) extends BlockId {
+  override def name: String = RemoteBlockPushResolver.MERGED_SHUFFLE_FILE_NAME_PREFIX + "_" +
+    appId + "_" + shuffleId + "_" + shuffleMergeId + "_" + reduceId + ".data"
+}
+
+@Since("3.2.0")
+@DeveloperApi
+case class ShuffleMergedIndexBlockId(
+    appId: String,
+    shuffleId: Int,
+    shuffleMergeId: Int,
+    reduceId: Int) extends BlockId {
+  override def name: String = RemoteBlockPushResolver.MERGED_SHUFFLE_FILE_NAME_PREFIX + "_" +
+    appId + "_" + shuffleId + "_" + shuffleMergeId + "_" + reduceId + ".index"
+}
+
+@Since("3.2.0")
+@DeveloperApi
+case class ShuffleMergedMetaBlockId(
+    appId: String,
+    shuffleId: Int,
+    shuffleMergeId: Int,
+    reduceId: Int) extends BlockId {
+  override def name: String = RemoteBlockPushResolver.MERGED_SHUFFLE_FILE_NAME_PREFIX + "_" +
+    appId + "_" + shuffleId + "_" + shuffleMergeId + "_" + reduceId + ".meta"
+}
+
 @DeveloperApi
 case class BroadcastBlockId(broadcastId: Long, field: String = "") extends BlockId {
   override def name: String = "broadcast_" + broadcastId + (if (field == "") "" else "_" + field)
@@ -115,6 +189,11 @@ private[spark] case class TestBlockId(id: String) extends BlockId {
 class UnrecognizedBlockId(name: String)
     extends SparkException(s"Failed to parse $name into a block ID")
 
+@DeveloperApi
+case class CacheId(userId: String, sessionId: String, hash: String) extends BlockId {
+  override def name: String = s"cache_${userId}_${sessionId}_$hash"
+}
+
 @DeveloperApi
 object BlockId {
   val RDD = "rdd_([0-9]+)_([0-9]+)".r
@@ -122,6 +201,15 @@ object BlockId {
   val SHUFFLE_BATCH = "shuffle_([0-9]+)_([0-9]+)_([0-9]+)_([0-9]+)".r
   val SHUFFLE_DATA = "shuffle_([0-9]+)_([0-9]+)_([0-9]+).data".r
   val SHUFFLE_INDEX = "shuffle_([0-9]+)_([0-9]+)_([0-9]+).index".r
+  val SHUFFLE_PUSH = "shufflePush_([0-9]+)_([0-9]+)_([0-9]+)_([0-9]+)".r
+  val SHUFFLE_MERGED = "shuffleMerged_([0-9]+)_([0-9]+)_([0-9]+)".r
+  val SHUFFLE_MERGED_DATA =
+    "shuffleMerged_([_A-Za-z0-9]*)_([0-9]+)_([0-9]+)_([0-9]+).data".r
+  val SHUFFLE_MERGED_INDEX =
+    "shuffleMerged_([_A-Za-z0-9]*)_([0-9]+)_([0-9]+)_([0-9]+).index".r
+  val SHUFFLE_MERGED_META =
+    "shuffleMerged_([_A-Za-z0-9]*)_([0-9]+)_([0-9]+)_([0-9]+).meta".r
+  val SHUFFLE_CHUNK = "shuffleChunk_([0-9]+)_([0-9]+)_([0-9]+)_([0-9]+)".r
   val BROADCAST = "broadcast_([0-9]+)([_A-Za-z0-9]*)".r
   val TASKRESULT = "taskresult_([0-9]+)".r
   val STREAM = "input-([0-9]+)-([0-9]+)".r
@@ -140,6 +228,22 @@ object BlockId {
       ShuffleDataBlockId(shuffleId.toInt, mapId.toLong, reduceId.toInt)
     case SHUFFLE_INDEX(shuffleId, mapId, reduceId) =>
       ShuffleIndexBlockId(shuffleId.toInt, mapId.toLong, reduceId.toInt)
+    case SHUFFLE_PUSH(shuffleId, shuffleMergeId, mapIndex, reduceId) =>
+      ShufflePushBlockId(shuffleId.toInt, shuffleMergeId.toInt, mapIndex.toInt,
+        reduceId.toInt)
+    case SHUFFLE_MERGED(shuffleId, shuffleMergeId, reduceId) =>
+      ShuffleMergedBlockId(shuffleId.toInt, shuffleMergeId.toInt, reduceId.toInt)
+    case SHUFFLE_MERGED_DATA(appId, shuffleId, shuffleMergeId, reduceId) =>
+      ShuffleMergedDataBlockId(appId, shuffleId.toInt, shuffleMergeId.toInt, reduceId.toInt)
+    case SHUFFLE_MERGED_INDEX(appId, shuffleId, shuffleMergeId, reduceId) =>
+      ShuffleMergedIndexBlockId(appId, shuffleId.toInt, shuffleMergeId.toInt,
+        reduceId.toInt)
+    case SHUFFLE_MERGED_META(appId, shuffleId, shuffleMergeId, reduceId) =>
+      ShuffleMergedMetaBlockId(appId, shuffleId.toInt, shuffleMergeId.toInt,
+        reduceId.toInt)
+    case SHUFFLE_CHUNK(shuffleId, shuffleMergeId, reduceId, chunkId) =>
+      ShuffleBlockChunkId(shuffleId.toInt, shuffleMergeId.toInt, reduceId.toInt,
+        chunkId.toInt)
     case BROADCAST(broadcastId, field) =>
       BroadcastBlockId(broadcastId.toLong, field.stripPrefix("_"))
     case TASKRESULT(taskId) =>
@@ -152,7 +256,6 @@ object BlockId {
       TempShuffleBlockId(UUID.fromString(uuid))
     case TEST(value) =>
       TestBlockId(value)
-    case _ =>
-      throw new UnrecognizedBlockId(name)
+    case _ => throw SparkCoreErrors.unrecognizedBlockIdError(name)
   }
 }
