@@ -31,20 +31,22 @@ import org.apache.spark.ui.UIUtils
 private[ui] class PoolTable(pools: Map[Schedulable, PoolData], parent: StagesTab) {
 
   def toNodeSeq(request: HttpServletRequest): Seq[Node] = {
-    <table class="table table-bordered table-striped table-condensed sortable table-fixed">
+    <table class="table table-bordered table-striped table-sm sortable table-fixed">
       <thead>
-        <th>Pool Name</th>
-        <th>
-          <span data-toggle="tooltip" data-placement="top" title="Pool's minimum share of CPU
-           cores">Minimum Share</span>
-        </th>
-        <th>
-          <span data-toggle="tooltip" data-placement="top" title="Pool's share of cluster resources
-           relative to others">Pool Weight</span>
-        </th>
-        <th>Active Stages</th>
-        <th>Running Tasks</th>
-        <th>Scheduling Mode</th>
+        <tr>
+          <th>Pool Name</th>
+          <th>
+            <span data-toggle="tooltip" data-placement="top" title="Pool's minimum share of CPU
+             cores">Minimum Share</span>
+          </th>
+          <th>
+            <span data-toggle="tooltip" data-placement="top" title="Pool's share of cluster
+             resources relative to others">Pool Weight</span>
+          </th>
+          <th>Active Stages</th>
+          <th>Running Tasks</th>
+          <th>SchedulingMode</th>
+        </tr>
       </thead>
       <tbody>
         {pools.map { case (s, p) => poolRow(request, s, p) }}
