@@ -26,6 +26,7 @@ import org.apache.spark.{SecurityManager, SparkConf, SparkFunSuite}
 import org.apache.spark.internal.config._
 import org.apache.spark.network.shuffle.{ExternalBlockHandler, ExternalShuffleBlockResolver}
 import org.apache.spark.network.shuffle.TestShuffleDataContext
+import org.apache.spark.network.shuffledb.DBBackend
 import org.apache.spark.tags.ExtendedLevelDBTest
 import org.apache.spark.util.Utils
 
@@ -34,8 +35,7 @@ import org.apache.spark.util.Utils
  * with #spark.shuffle.service.db.enabled = true or false
  * Note that failures in this suite may arise when#spark.shuffle.service.db.enabled = false
  */
-@ExtendedLevelDBTest
-class ExternalShuffleServiceDbSuite extends SparkFunSuite {
+abstract class ExternalShuffleServiceDbSuite extends SparkFunSuite {
   val sortBlock0 = "Hello!"
   val sortBlock1 = "World!"
   val SORT_MANAGER = "org.apache.spark.shuffle.sort.SortShuffleManager"
@@ -48,6 +48,8 @@ class ExternalShuffleServiceDbSuite extends SparkFunSuite {
   var blockHandler: ExternalBlockHandler = _
   var blockResolver: ExternalShuffleBlockResolver = _
 
+  protected def shuffleDBBackend(): DBBackend
+
   override def beforeAll(): Unit = {
     super.beforeAll()
     sparkConf = new SparkConf()
@@ -78,6 +80,7 @@ class ExternalShuffleServiceDbSuite extends SparkFunSuite {
   def registerExecutor(): Unit = {
     try {
       sparkConf.set("spark.shuffle.service.db.enabled", "true")
+      sparkConf.set(SHUFFLE_SERVICE_DB_BACKEND.key, shuffleDBBackend().name())
       externalShuffleService = new ExternalShuffleService(shuffleServiceConf, securityManager)
 
       // external Shuffle Service start
@@ -99,6 +102,7 @@ class ExternalShuffleServiceDbSuite extends SparkFunSuite {
     "shuffle service restart") {
     try {
       sparkConf.set("spark.shuffle.service.db.enabled", "true")
+      sparkConf.set(SHUFFLE_SERVICE_DB_BACKEND.key, shuffleDBBackend().name())
       externalShuffleService = new ExternalShuffleService(shuffleServiceConf, securityManager)
       // externalShuffleService restart
       externalShuffleService.start()
@@ -143,3 +147,12 @@ class ExternalShuffleServiceDbSuite extends SparkFunSuite {
     }
   }
 }
+
+@ExtendedLevelDBTest
+class ExternalShuffleServiceLevelDBSuite extends ExternalShuffleServiceDbSuite {
+  override protected def shuffleDBBackend(): DBBackend = DBBackend.LEVELDB
+}
+
+class ExternalShuffleServiceRocksDBSuite extends ExternalShuffleServiceDbSuite {
+  override protected def shuffleDBBackend(): DBBackend = DBBackend.ROCKSDB
+}
