@@ -17,7 +17,7 @@
 
 package org.apache.spark.serializer
 
-import java.io.{ByteArrayInputStream, ByteArrayOutputStream}
+import java.io.{ByteArrayInputStream, ByteArrayOutputStream, EOFException}
 import java.nio.ByteBuffer
 import java.util.concurrent.Executors
 
@@ -551,6 +551,17 @@ class KryoSerializerSuite extends SparkFunSuite with SharedSparkContext {
     val set = new OpenHashMap[Double, Double](10)
     ser.serialize(set)
   }
+
+  test("SPARK-43898: Register scala.collection.immutable.ArraySeq$ofRef for Scala 2.13") {
+    assume(scala.util.Properties.versionNumberString.startsWith("2.13"))
+    val conf = new SparkConf(false)
+    conf.set(KRYO_REGISTRATION_REQUIRED, true)
+    val ser = new KryoSerializer(conf).newInstance()
+    def check[T: ClassTag](t: T): Unit = {
+      assert(ser.deserialize[T](ser.serialize(t)) === t)
+    }
+    check(Utils.classForName("scala.collection.immutable.ArraySeq$ofRef"))
+  }
 }
 
 class KryoSerializerAutoResetDisabledSuite extends SparkFunSuite with SharedSparkContext {
@@ -598,6 +609,28 @@ class KryoSerializerAutoResetDisabledSuite extends SparkFunSuite with SharedSpar
     assert(serInstance.deserialize[Any](serObj) === (obj))
     assert(serInstance.deserialize[Any](byteBuffer) === (obj))
   }
+
+  test("SPARK-40912: Ignore unexpectedly truncated buffer") {
+    // This test checks that the improvement in SPARK-40912 does not break backwards compatabillity.
+    // But the behvaior of the asIterator iterface of silently ignoring trucated data should be
+    // revisited in a follow up ticket.
+    val serInstance = new KryoSerializer(conf).newInstance().asInstanceOf[KryoSerializerInstance]
+    val serialized: Array[Byte] = {
+      val baos = new ByteArrayOutputStream()
+      val serStream = serInstance.serializeStream(baos)
+      serStream.writeObject(KryoTest.CaseClass(0, ""))
+      serStream.close()
+      baos.toByteArray
+    }
+    // Make sure we disregard some data
+    assert(serialized.length > 2)
+    val trucated = serialized.take(2).toArray
+    val deserializationStream = serInstance.deserializeStream(new ByteArrayInputStream(trucated))
+    intercept[EOFException](
+      serInstance.deserializeStream(new ByteArrayInputStream(trucated)).readValue()
+    )
+    assert(deserializationStream.asIterator.toSeq == Seq())
+  }
 }
 
 class ClassLoaderTestingObject
