@@ -33,14 +33,19 @@ import org.apache.spark.storage.memory.{BlockEvictionHandler, MemoryStore, Parti
 import org.apache.spark.util._
 import org.apache.spark.util.io.ChunkedByteBuffer
 
+case class OffHeapValue(override val estimatedSize: Long) extends KnownSizeEstimation
+
 class MemoryStoreSuite
   extends SparkFunSuite
   with PrivateMethodTester
   with BeforeAndAfterEach
   with ResetSystemProperties {
 
-  var conf: SparkConf = new SparkConf(false)
+  val conf: SparkConf = new SparkConf(false)
     .set(STORAGE_UNROLL_MEMORY_THRESHOLD, 512L)
+    .set(MEMORY_OFFHEAP_ENABLED, true)
+    .set(MEMORY_OFFHEAP_SIZE, 12000L)
+    .set(MEMORY_STORAGE_FRACTION, 0.5)
 
   // Reuse a serializer across tests to avoid creating a new thread-local buffer on each test
   val serializer = new KryoSerializer(
@@ -156,7 +161,7 @@ class MemoryStoreSuite
       assert(blockInfoManager.lockNewBlockForWriting(
         blockId,
         new BlockInfo(StorageLevel.MEMORY_ONLY, classTag, tellMaster = false)))
-      val res = memoryStore.putIteratorAsValues(blockId, iter, classTag)
+      val res = memoryStore.putIteratorAsValues(blockId, iter, MemoryMode.ON_HEAP, classTag)
       blockInfoManager.unlock(blockId)
       res
     }
@@ -196,10 +201,13 @@ class MemoryStoreSuite
     assert(memoryStore.currentUnrollMemoryForThisTask === 0)
   }
 
-  test("safely unroll blocks through putIteratorAsValues") {
+  def testPutIteratorAsValues[T](
+      smallListValue: () => T,
+      bigListValue: () => T,
+      storageLevel: StorageLevel): Unit = {
     val (memoryStore, blockInfoManager) = makeMemoryStore(12000)
-    val smallList = List.fill(40)(new Array[Byte](100))
-    val bigList = List.fill(40)(new Array[Byte](1000))
+    val smallList = List.fill(40)(smallListValue())
+    val bigList = List.fill(40)(bigListValue())
     def smallIterator: Iterator[Any] = smallList.iterator.asInstanceOf[Iterator[Any]]
     def bigIterator: Iterator[Any] = bigList.iterator.asInstanceOf[Iterator[Any]]
     assert(memoryStore.currentUnrollMemoryForThisTask === 0)
@@ -210,8 +218,8 @@ class MemoryStoreSuite
         classTag: ClassTag[T]): Either[PartiallyUnrolledIterator[T], Long] = {
       assert(blockInfoManager.lockNewBlockForWriting(
         blockId,
-        new BlockInfo(StorageLevel.MEMORY_ONLY, classTag, tellMaster = false)))
-      val res = memoryStore.putIteratorAsValues(blockId, iter, classTag)
+        new BlockInfo(storageLevel, classTag, tellMaster = false)))
+      val res = memoryStore.putIteratorAsValues(blockId, iter, storageLevel.memoryMode, classTag)
       blockInfoManager.unlock(blockId)
       res
     }
@@ -260,6 +268,18 @@ class MemoryStoreSuite
     assert(memoryStore.currentUnrollMemoryForThisTask === 0) // close released the unroll memory
   }
 
+  test("safely unroll blocks through putIteratorAsValues") {
+    testPutIteratorAsValues(() => new Array[Byte](100), () => new Array[Byte](1000),
+      StorageLevel.MEMORY_ONLY)
+  }
+
+  test("safely unroll blocks through putIteratorAsValues off-heap") {
+    // Size the values so the sequence of block drops matches the one in the on-heap test. The
+    // values are different since the sizes here are exact, vs. being estimated for on-heap.
+    testPutIteratorAsValues(() => OffHeapValue(110), () => OffHeapValue(1500),
+      StorageLevel(false, true, useOffHeap = true, deserialized = true, 1))
+  }
+
   test("safely unroll blocks through putIteratorAsBytes") {
     val (memoryStore, blockInfoManager) = makeMemoryStore(8400)
     val smallList = List.fill(40)(new Array[Byte](100))
@@ -376,7 +396,7 @@ class MemoryStoreSuite
     def putIteratorAsValues(
         blockId: BlockId,
         iter: Iterator[Any]): Either[PartiallyUnrolledIterator[Any], Long] = {
-       memoryStore.putIteratorAsValues(blockId, iter, ClassTag.Any)
+       memoryStore.putIteratorAsValues(blockId, iter, MemoryMode.ON_HEAP, ClassTag.Any)
     }
 
     // All unroll memory used is released because putIterator did not return an iterator
@@ -546,4 +566,78 @@ class MemoryStoreSuite
       }
     }
   }
+
+  test("put user-defined objects to MemoryStore and remove") {
+    val (memoryStore, _) = makeMemoryStore(12000)
+    val allocator = DummyAllocator()
+    val nativeObjList = List.fill(40)(NativeObject(allocator, 100))
+    def nativeObjIterator: Iterator[Any] = nativeObjList.iterator.asInstanceOf[Iterator[Any]]
+    def putIteratorAsValues[T](
+        blockId: BlockId,
+        iter: Iterator[T],
+        classTag: ClassTag[T]): Either[PartiallyUnrolledIterator[T], Long] = {
+      memoryStore.putIteratorAsValues(blockId, iter, MemoryMode.ON_HEAP, classTag)
+    }
+
+    // Unroll with plenty of space. This should succeed and cache both blocks.
+    assert(putIteratorAsValues("b1", nativeObjIterator, ClassTag.Any).isRight)
+    assert(putIteratorAsValues("b2", nativeObjIterator, ClassTag.Any).isRight)
+
+    memoryStore.remove("b1")
+    memoryStore.remove("b2")
+
+    // Check if allocator was cleared.
+    while (allocator.getAllocatedMemory > 0) {
+      Thread.sleep(500)
+    }
+    assert(allocator.getAllocatedMemory == 0)
+  }
+
+  test("put user-defined objects to MemoryStore and clear") {
+    val (memoryStore, _) = makeMemoryStore(12000)
+    val allocator = DummyAllocator()
+    val nativeObjList = List.fill(40)(NativeObject(allocator, 100))
+    def nativeObjIterator: Iterator[Any] = nativeObjList.iterator.asInstanceOf[Iterator[Any]]
+    def putIteratorAsValues[T](
+        blockId: BlockId,
+        iter: Iterator[T],
+        classTag: ClassTag[T]): Either[PartiallyUnrolledIterator[T], Long] = {
+      memoryStore.putIteratorAsValues(blockId, iter, MemoryMode.ON_HEAP, classTag)
+    }
+
+    // Unroll with plenty of space. This should succeed and cache both blocks.
+    assert(putIteratorAsValues("b1", nativeObjIterator, ClassTag.Any).isRight)
+    assert(putIteratorAsValues("b2", nativeObjIterator, ClassTag.Any).isRight)
+
+    memoryStore.clear
+    // Check if allocator was cleared.
+    while (allocator.getAllocatedMemory > 0) {
+      Thread.sleep(500)
+    }
+    assert(allocator.getAllocatedMemory == 0)
+  }
+}
+
+private case class DummyAllocator() {
+  private var allocated: Int = 0
+  def alloc(size: Int): Unit = synchronized {
+    allocated += size
+  }
+  def release(size: Int): Unit = synchronized {
+    allocated -= size
+  }
+  def getAllocatedMemory: Int = synchronized {
+    allocated
+  }
+}
+
+private case class NativeObject(alloc: DummyAllocator, size: Int) extends KnownSizeEstimation
+  with AutoCloseable {
+  alloc.alloc(size)
+  var allocated_size: Int = size
+  override def estimatedSize: Long = allocated_size
+  override def close(): Unit = synchronized {
+    alloc.release(allocated_size)
+    allocated_size = 0
+  }
 }
