@@ -87,6 +87,10 @@ class LDASuite extends MLTest with DefaultReadWriteTest {
     assert(lda.getTopicDistributionCol === "topicDistribution")
   }
 
+  test("LDA validate input dataset") {
+    testInvalidVectors(new LDA().fit(_))
+  }
+
   test("set parameters") {
     val lda = new LDA()
       .setFeaturesCol("test_feature")
@@ -199,11 +203,11 @@ class LDASuite extends MLTest with DefaultReadWriteTest {
     assert(topics.count() === k)
     assert(topics.select("topic").rdd.map(_.getInt(0)).collect().toSet === Range(0, k).toSet)
     topics.select("termIndices").collect().foreach { case r: Row =>
-      val termIndices = r.getAs[Seq[Int]](0)
+      val termIndices = r.getSeq[Int](0)
       assert(termIndices.length === 3 && termIndices.toSet.size === 3)
     }
     topics.select("termWeights").collect().foreach { case r: Row =>
-      val termWeights = r.getAs[Seq[Double]](0)
+      val termWeights = r.getSeq[Double](0)
       assert(termWeights.length === 3 && termWeights.forall(w => w >= 0.0 && w <= 1.0))
     }
   }
