@@ -19,7 +19,7 @@ package org.apache.spark.deploy.k8s
 
 import io.fabric8.kubernetes.api.model.{LocalObjectReferenceBuilder, PodBuilder}
 
-import org.apache.spark.{SparkConf, SparkFunSuite}
+import org.apache.spark.{SPARK_VERSION, SparkConf, SparkFunSuite}
 import org.apache.spark.deploy.k8s.Config._
 import org.apache.spark.deploy.k8s.Constants._
 import org.apache.spark.deploy.k8s.submit._
@@ -28,6 +28,15 @@ import org.apache.spark.resource.ResourceProfile.DEFAULT_RESOURCE_PROFILE_ID
 class KubernetesConfSuite extends SparkFunSuite {
 
   private val APP_ARGS = Array("arg1", "arg2")
+  private val CUSTOM_NODE_SELECTOR = Map(
+    "nodeSelectorKey1" -> "nodeSelectorValue1",
+    "nodeSelectorKey2" -> "nodeSelectorValue2")
+  private val CUSTOM_DRIVER_NODE_SELECTOR = Map(
+    "driverNodeSelectorKey1" -> "driverNodeSelectorValue1",
+    "driverNodeSelectorKey2" -> "driverNodeSelectorValue2")
+  private val CUSTOM_EXECUTOR_NODE_SELECTOR = Map(
+    "execNodeSelectorKey1" -> "execNodeSelectorValue1",
+    "execNodeSelectorKey2" -> "execNodeSelectorValue2")
   private val CUSTOM_LABELS = Map(
     "customLabel1Key" -> "customLabel1Value",
     "customLabel2Key" -> "customLabel2Value")
@@ -79,7 +88,9 @@ class KubernetesConfSuite extends SparkFunSuite {
       APP_ARGS,
       None)
     assert(conf.labels === Map(
+      SPARK_VERSION_LABEL -> SPARK_VERSION,
       SPARK_APP_ID_LABEL -> KubernetesTestConf.APP_ID,
+      SPARK_APP_NAME_LABEL -> KubernetesConf.getAppNameLabel(conf.appName),
       SPARK_ROLE_LABEL -> SPARK_POD_DRIVER_ROLE) ++
       CUSTOM_LABELS)
     assert(conf.annotations === CUSTOM_ANNOTATIONS)
@@ -144,8 +155,10 @@ class KubernetesConfSuite extends SparkFunSuite {
       KubernetesTestConf.APP_ID,
       Some(DRIVER_POD))
     assert(conf.labels === Map(
+      SPARK_VERSION_LABEL -> SPARK_VERSION,
       SPARK_EXECUTOR_ID_LABEL -> EXECUTOR_ID,
       SPARK_APP_ID_LABEL -> KubernetesTestConf.APP_ID,
+      SPARK_APP_NAME_LABEL -> KubernetesConf.getAppNameLabel(conf.appName),
       SPARK_ROLE_LABEL -> SPARK_POD_EXECUTOR_ROLE,
       SPARK_RESOURCE_PROFILE_ID_LABEL -> DEFAULT_RESOURCE_PROFILE_ID.toString) ++ CUSTOM_LABELS)
     assert(conf.annotations === CUSTOM_ANNOTATIONS)
@@ -171,12 +184,71 @@ class KubernetesConfSuite extends SparkFunSuite {
         "executorEnvVars5-var5" -> "executorEnvVars5/var5"))
   }
 
+  test("SPARK-36075: Set nodeSelector, driverNodeSelector, executorNodeSelect") {
+    val sparkConf = new SparkConf(false)
+    CUSTOM_NODE_SELECTOR.foreach { case (key, value) =>
+      sparkConf.set(s"$KUBERNETES_NODE_SELECTOR_PREFIX$key", value)
+    }
+    CUSTOM_DRIVER_NODE_SELECTOR.foreach { case (key, value) =>
+      sparkConf.set(s"$KUBERNETES_DRIVER_NODE_SELECTOR_PREFIX$key", value)
+    }
+    CUSTOM_EXECUTOR_NODE_SELECTOR.foreach { case (key, value) =>
+      sparkConf.set(s"$KUBERNETES_EXECUTOR_NODE_SELECTOR_PREFIX$key", value)
+    }
+    val execConf = KubernetesTestConf.createExecutorConf(sparkConf)
+    assert(execConf.nodeSelector === CUSTOM_NODE_SELECTOR)
+    assert(execConf.executorNodeSelector === CUSTOM_EXECUTOR_NODE_SELECTOR)
+    val driverConf = KubernetesTestConf.createDriverConf(sparkConf)
+    assert(driverConf.nodeSelector === CUSTOM_NODE_SELECTOR)
+    assert(driverConf.driverNodeSelector === CUSTOM_DRIVER_NODE_SELECTOR)
+  }
+
+  test("SPARK-36059: Set driver.scheduler and executor.scheduler") {
+    val sparkConf = new SparkConf(false)
+    val execUnsetConf = KubernetesTestConf.createExecutorConf(sparkConf)
+    val driverUnsetConf = KubernetesTestConf.createDriverConf(sparkConf)
+    assert(execUnsetConf.schedulerName === None)
+    assert(driverUnsetConf.schedulerName === None)
+
+    sparkConf.set(KUBERNETES_SCHEDULER_NAME, "sameScheduler")
+    // Use KUBERNETES_SCHEDULER_NAME when is NOT set
+    assert(KubernetesTestConf.createDriverConf(sparkConf).schedulerName === Some("sameScheduler"))
+    assert(KubernetesTestConf.createExecutorConf(sparkConf).schedulerName === Some("sameScheduler"))
+
+    // Override by driver/executor side scheduler when ""
+    sparkConf.set(KUBERNETES_DRIVER_SCHEDULER_NAME, "")
+    sparkConf.set(KUBERNETES_EXECUTOR_SCHEDULER_NAME, "")
+    assert(KubernetesTestConf.createDriverConf(sparkConf).schedulerName === Some(""))
+    assert(KubernetesTestConf.createExecutorConf(sparkConf).schedulerName === Some(""))
+
+    // Override by driver/executor side scheduler when set
+    sparkConf.set(KUBERNETES_DRIVER_SCHEDULER_NAME, "driverScheduler")
+    sparkConf.set(KUBERNETES_EXECUTOR_SCHEDULER_NAME, "executorScheduler")
+    val execConf = KubernetesTestConf.createExecutorConf(sparkConf)
+    assert(execConf.schedulerName === Some("executorScheduler"))
+    val driverConf = KubernetesTestConf.createDriverConf(sparkConf)
+    assert(driverConf.schedulerName === Some("driverScheduler"))
+  }
+
   test("SPARK-37735: access appId in KubernetesConf") {
     val sparkConf = new SparkConf(false)
     val driverConf = KubernetesTestConf.createDriverConf(sparkConf)
     val execConf = KubernetesTestConf.createExecutorConf(sparkConf)
-    assert(driverConf.asInstanceOf[KubernetesConf].appId === KubernetesTestConf.APP_ID)
-    assert(execConf.asInstanceOf[KubernetesConf].appId === KubernetesTestConf.APP_ID)
+    assert(driverConf.appId === KubernetesTestConf.APP_ID)
+    assert(execConf.appId === KubernetesTestConf.APP_ID)
+  }
+
+  test("SPARK-36566: get app name label") {
+    assert(KubernetesConf.getAppNameLabel(" Job+Spark-Pi 2021") === "job-spark-pi-2021")
+    assert(KubernetesConf.getAppNameLabel("a" * 63) === "a" * 63)
+    assert(KubernetesConf.getAppNameLabel("a" * 64) === "a" * 63)
+    assert(KubernetesConf.getAppNameLabel("a" * 253) === "a" * 63)
+  }
+
+  test("SPARK-38630: K8s label value should start and end with alphanumeric") {
+    assert(KubernetesConf.getAppNameLabel("-hello-") === "hello")
+    assert(KubernetesConf.getAppNameLabel("a" * 62 + "-aaa") === "a" * 62)
+    assert(KubernetesConf.getAppNameLabel("-" + "a" * 63) === "a" * 62)
   }
 
   test("SPARK-40869: Resource name prefix should not start with a hyphen") {
