@@ -367,12 +367,12 @@ case class InSubquery(values: Seq[Expression], query: ListQuery)
   final override val nodePatterns: Seq[TreePattern] = Seq(IN_SUBQUERY)
 
   override def checkInputDataTypes(): TypeCheckResult = {
-    if (values.length != query.childOutputs.length) {
+    if (values.length != query.numCols) {
       DataTypeMismatch(
         errorSubClass = "IN_SUBQUERY_LENGTH_MISMATCH",
         messageParameters = Map(
           "leftLength" -> values.length.toString,
-          "rightLength" -> query.childOutputs.length.toString,
+          "rightLength" -> query.numCols.toString,
           "leftColumns" -> values.map(toSQLExpr(_)).mkString(", "),
           "rightColumns" -> query.childOutputs.map(toSQLExpr(_)).mkString(", ")
         )
@@ -399,7 +399,15 @@ case class InSubquery(values: Seq[Expression], query: ListQuery)
   }
 
   override def children: Seq[Expression] = values :+ query
-  override def nullable: Boolean = children.exists(_.nullable)
+  override def nullable: Boolean = {
+    if (!SQLConf.get.getConf(SQLConf.LEGACY_IN_SUBQUERY_NULLABILITY)) {
+      values.exists(_.nullable) || query.childOutputs.exists(_.nullable)
+    } else {
+      // Legacy (incorrect) behavior checked only the nullability of the left-hand side
+      // (see SPARK-43413).
+      values.exists(_.nullable)
+    }
+  }
   override def toString: String = s"$value IN ($query)"
   override def sql: String = s"(${value.sql} IN (${query.sql}))"
 
@@ -460,6 +468,8 @@ case class In(value: Expression, list: Seq[Expression]) extends Predicate {
   override def foldable: Boolean = children.forall(_.foldable)
 
   final override val nodePatterns: Seq[TreePattern] = Seq(IN)
+  private val legacyNullInEmptyBehavior =
+    SQLConf.get.getConf(SQLConf.LEGACY_NULL_IN_EMPTY_LIST_BEHAVIOR)
 
   override lazy val canonicalized: Expression = {
     val basic = withNewChildren(children.map(_.canonicalized)).asInstanceOf[In]
@@ -473,88 +483,104 @@ case class In(value: Expression, list: Seq[Expression]) extends Predicate {
   override def toString: String = s"$value IN ${list.mkString("(", ",", ")")}"
 
   override def eval(input: InternalRow): Any = {
-    val evaluatedValue = value.eval(input)
-    if (evaluatedValue == null) {
-      null
+    if (list.isEmpty && !legacyNullInEmptyBehavior) {
+      // IN (empty list) is always false under current behavior.
+      // Under legacy behavior it's null if the left side is null, otherwise false (SPARK-44550).
+      false
     } else {
-      var hasNull = false
-      list.foreach { e =>
-        val v = e.eval(input)
-        if (v == null) {
-          hasNull = true
-        } else if (ordering.equiv(v, evaluatedValue)) {
-          return true
-        }
-      }
-      if (hasNull) {
+      val evaluatedValue = value.eval(input)
+      if (evaluatedValue == null) {
         null
       } else {
-        false
+        var hasNull = false
+        list.foreach { e =>
+          val v = e.eval(input)
+          if (v == null) {
+            hasNull = true
+          } else if (ordering.equiv(v, evaluatedValue)) {
+            return true
+          }
+        }
+        if (hasNull) {
+          null
+        } else {
+          false
+        }
       }
     }
   }
 
   override def doGenCode(ctx: CodegenContext, ev: ExprCode): ExprCode = {
-    val javaDataType = CodeGenerator.javaType(value.dataType)
-    val valueGen = value.genCode(ctx)
-    val listGen = list.map(_.genCode(ctx))
-    // inTmpResult has 3 possible values:
-    // -1 means no matches found and there is at least one value in the list evaluated to null
-    val HAS_NULL = -1
-    // 0 means no matches found and all values in the list are not null
-    val NOT_MATCHED = 0
-    // 1 means one value in the list is matched
-    val MATCHED = 1
-    val tmpResult = ctx.freshName("inTmpResult")
-    val valueArg = ctx.freshName("valueArg")
-    // All the blocks are meant to be inside a do { ... } while (false); loop.
-    // The evaluation of variables can be stopped when we find a matching value.
-    val listCode = listGen.map(x =>
-      s"""
-         |${x.code}
-         |if (${x.isNull}) {
-         |  $tmpResult = $HAS_NULL; // ${ev.isNull} = true;
-         |} else if (${ctx.genEqual(value.dataType, valueArg, x.value)}) {
-         |  $tmpResult = $MATCHED; // ${ev.isNull} = false; ${ev.value} = true;
-         |  continue;
-         |}
+    if (list.isEmpty && !legacyNullInEmptyBehavior) {
+      // IN (empty list) is always false under current behavior.
+      // Under legacy behavior it's null if the left side is null, otherwise false (SPARK-44550).
+      ev.copy(code =
+        code"""
+              |final boolean ${ev.isNull} = false;
+              |final boolean ${ev.value} = false;
        """.stripMargin)
-
-    val codes = ctx.splitExpressionsWithCurrentInputs(
-      expressions = listCode,
-      funcName = "valueIn",
-      extraArguments = (javaDataType, valueArg) :: (CodeGenerator.JAVA_BYTE, tmpResult) :: Nil,
-      returnType = CodeGenerator.JAVA_BYTE,
-      makeSplitFunction = body =>
+    } else {
+      val javaDataType = CodeGenerator.javaType(value.dataType)
+      val valueGen = value.genCode(ctx)
+      val listGen = list.map(_.genCode(ctx))
+      // inTmpResult has 3 possible values:
+      // -1 means no matches found and there is at least one value in the list evaluated to null
+      val HAS_NULL = -1
+      // 0 means no matches found and all values in the list are not null
+      val NOT_MATCHED = 0
+      // 1 means one value in the list is matched
+      val MATCHED = 1
+      val tmpResult = ctx.freshName("inTmpResult")
+      val valueArg = ctx.freshName("valueArg")
+      // All the blocks are meant to be inside a do { ... } while (false); loop.
+      // The evaluation of variables can be stopped when we find a matching value.
+      val listCode = listGen.map(x =>
         s"""
-           |do {
-           |  $body
-           |} while (false);
-           |return $tmpResult;
-         """.stripMargin,
-      foldFunctions = _.map { funcCall =>
-        s"""
-           |$tmpResult = $funcCall;
-           |if ($tmpResult == $MATCHED) {
+           |${x.code}
+           |if (${x.isNull}) {
+           |  $tmpResult = $HAS_NULL; // ${ev.isNull} = true;
+           |} else if (${ctx.genEqual(value.dataType, valueArg, x.value)}) {
+           |  $tmpResult = $MATCHED; // ${ev.isNull} = false; ${ev.value} = true;
            |  continue;
            |}
-         """.stripMargin
-      }.mkString("\n"))
+         """.stripMargin)
 
-    ev.copy(code =
-      code"""
-         |${valueGen.code}
-         |byte $tmpResult = $HAS_NULL;
-         |if (!${valueGen.isNull}) {
-         |  $tmpResult = $NOT_MATCHED;
-         |  $javaDataType $valueArg = ${valueGen.value};
-         |  do {
-         |    $codes
-         |  } while (false);
-         |}
-         |final boolean ${ev.isNull} = ($tmpResult == $HAS_NULL);
-         |final boolean ${ev.value} = ($tmpResult == $MATCHED);
-       """.stripMargin)
+      val codes = ctx.splitExpressionsWithCurrentInputs(
+        expressions = listCode,
+        funcName = "valueIn",
+        extraArguments = (javaDataType, valueArg) :: (CodeGenerator.JAVA_BYTE, tmpResult) :: Nil,
+        returnType = CodeGenerator.JAVA_BYTE,
+        makeSplitFunction = body =>
+          s"""
+             |do {
+             |  $body
+             |} while (false);
+             |return $tmpResult;
+           """.stripMargin,
+        foldFunctions = _.map { funcCall =>
+          s"""
+             |$tmpResult = $funcCall;
+             |if ($tmpResult == $MATCHED) {
+             |  continue;
+             |}
+           """.stripMargin
+        }.mkString("\n"))
+
+      ev.copy(code =
+        code"""
+           |${valueGen.code}
+           |byte $tmpResult = $HAS_NULL;
+           |if (!${valueGen.isNull}) {
+           |  $tmpResult = $NOT_MATCHED;
+           |  $javaDataType $valueArg = ${valueGen.value};
+           |  do {
+           |    $codes
+           |  } while (false);
+           |}
+           |final boolean ${ev.isNull} = ($tmpResult == $HAS_NULL);
+           |final boolean ${ev.value} = ($tmpResult == $MATCHED);
+         """.stripMargin)
+    }
   }
 
   override def sql: String = {
@@ -599,16 +625,27 @@ case class InSet(child: Expression, hset: Set[Any]) extends UnaryExpression with
   override def nullable: Boolean = child.nullable || hasNull
 
   final override val nodePatterns: Seq[TreePattern] = Seq(INSET)
+  private val legacyNullInEmptyBehavior =
+    SQLConf.get.getConf(SQLConf.LEGACY_NULL_IN_EMPTY_LIST_BEHAVIOR)
 
-  protected override def nullSafeEval(value: Any): Any = {
-    if (set.contains(value)) {
-      true
-    } else if (isNaN(value)) {
-      hasNaN
-    } else if (hasNull) {
-      null
-    } else {
+  override def eval(input: InternalRow): Any = {
+    if (hset.isEmpty && !legacyNullInEmptyBehavior) {
+      // IN (empty list) is always false under current behavior.
+      // Under legacy behavior it's null if the left side is null, otherwise false (SPARK-44550).
       false
+    } else {
+      val value = child.eval(input)
+      if (value == null) {
+        null
+      } else if (set.contains(value)) {
+        true
+      } else if (isNaN(value)) {
+        hasNaN
+      } else if (hasNull) {
+        null
+      } else {
+        false
+      }
     }
   }
 
@@ -621,7 +658,16 @@ case class InSet(child: Expression, hset: Set[Any]) extends UnaryExpression with
   }
 
   override def doGenCode(ctx: CodegenContext, ev: ExprCode): ExprCode = {
-    if (canBeComputedUsingSwitch && hset.size <= SQLConf.get.optimizerInSetSwitchThreshold) {
+    if (hset.isEmpty && !legacyNullInEmptyBehavior) {
+      // IN (empty list) is always false under current behavior.
+      // Under legacy behavior it's null if the left side is null, otherwise false (SPARK-44550).
+      ev.copy(code =
+        code"""
+        ${CodeGenerator.JAVA_BOOLEAN} ${ev.value} = false;
+        ${CodeGenerator.JAVA_BOOLEAN} ${ev.isNull} = false;
+        """
+      )
+    } else if (canBeComputedUsingSwitch && hset.size <= SQLConf.get.optimizerInSetSwitchThreshold) {
       genCodeWithSwitch(ctx, ev)
     } else {
       genCodeWithSet(ctx, ev)
