@@ -17,7 +17,7 @@
 
 package org.apache.spark.sql.execution.streaming
 
-import scala.collection.immutable
+import scala.collection.{immutable, GenTraversableOnce}
 
 import org.apache.spark.sql.connector.read.streaming.{Offset => OffsetV2, SparkDataStream}
 
@@ -29,7 +29,7 @@ class StreamProgress(
         new immutable.HashMap[SparkDataStream, OffsetV2])
   extends scala.collection.immutable.Map[SparkDataStream, OffsetV2] {
 
-  //  Note: this class supports Scala 2.13. A parallel source tree has a 2.12 implementation.
+  //  Note: this class supports Scala 2.12. A parallel source tree has a 2.13 implementation.
 
   def toOffsetSeq(source: Seq[SparkDataStream], metadata: OffsetSeqMetadata): OffsetSeq = {
     OffsetSeq(source.map(get), Some(metadata))
@@ -38,16 +38,17 @@ class StreamProgress(
   override def toString: String =
     baseMap.map { case (k, v) => s"$k: $v"}.mkString("{", ",", "}")
 
-  override def updated[B1 >: OffsetV2](key: SparkDataStream, value: B1): Map[SparkDataStream, B1] =
-    baseMap + (key -> value)
+  override def +[B1 >: OffsetV2](kv: (SparkDataStream, B1)): Map[SparkDataStream, B1] = {
+    baseMap + kv
+  }
 
   override def get(key: SparkDataStream): Option[OffsetV2] = baseMap.get(key)
 
   override def iterator: Iterator[(SparkDataStream, OffsetV2)] = baseMap.iterator
 
-  override def removed(key: SparkDataStream): Map[SparkDataStream, OffsetV2] = baseMap - key
+  override def -(key: SparkDataStream): Map[SparkDataStream, OffsetV2] = baseMap - key
 
-  def ++(updates: IterableOnce[(SparkDataStream, OffsetV2)]): StreamProgress = {
+  def ++(updates: GenTraversableOnce[(SparkDataStream, OffsetV2)]): StreamProgress = {
     new StreamProgress(baseMap ++ updates)
   }
 }
