@@ -17,7 +17,6 @@
 
 package org.apache.spark.sql.connector
 
-import java.util
 import java.util.Collections
 
 import test.org.apache.spark.sql.connector.catalog.functions._
@@ -25,7 +24,6 @@ import test.org.apache.spark.sql.connector.catalog.functions.JavaLongAdd._
 import test.org.apache.spark.sql.connector.catalog.functions.JavaRandomAdd._
 import test.org.apache.spark.sql.connector.catalog.functions.JavaStrLen._
 
-import org.apache.spark.SparkException
 import org.apache.spark.sql.{AnalysisException, DataFrame, Row}
 import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.expressions.CodegenObjectFactoryMode.{FALLBACK, NO_CODEGEN}
@@ -36,27 +34,203 @@ import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.types._
 import org.apache.spark.unsafe.types.UTF8String
 
+object IntAverage extends AggregateFunction[(Int, Int), Int] {
+  override def name(): String = "iavg"
+  override def canonicalName(): String = "h2.iavg"
+  override def inputTypes(): Array[DataType] = Array(IntegerType)
+  override def resultType(): DataType = IntegerType
+
+  override def newAggregationState(): (Int, Int) = (0, 0)
+
+  override def update(state: (Int, Int), input: InternalRow): (Int, Int) = {
+    if (input.isNullAt(0)) {
+      state
+    } else {
+      val i = input.getInt(0)
+      state match {
+        case (_, 0) =>
+          (i, 1)
+        case (total, count) =>
+          (total + i, count + 1)
+      }
+    }
+  }
+
+  override def merge(leftState: (Int, Int), rightState: (Int, Int)): (Int, Int) = {
+    (leftState._1 + rightState._1, leftState._2 + rightState._2)
+  }
+
+  override def produceResult(state: (Int, Int)): Int = state._1 / state._2
+}
+
+object LongAverage extends AggregateFunction[(Long, Long), Long] {
+  override def name(): String = "iavg"
+  override def canonicalName(): String = "h2.iavg"
+  override def inputTypes(): Array[DataType] = Array(LongType)
+  override def resultType(): DataType = LongType
+
+  override def newAggregationState(): (Long, Long) = (0L, 0L)
+
+  override def update(state: (Long, Long), input: InternalRow): (Long, Long) = {
+    if (input.isNullAt(0)) {
+      state
+    } else {
+      val l = input.getLong(0)
+      state match {
+        case (_, 0L) =>
+          (l, 1)
+        case (total, count) =>
+          (total + l, count + 1L)
+      }
+    }
+  }
+
+  override def merge(leftState: (Long, Long), rightState: (Long, Long)): (Long, Long) = {
+    (leftState._1 + rightState._1, leftState._2 + rightState._2)
+  }
+
+  override def produceResult(state: (Long, Long)): Long = state._1 / state._2
+}
+
+object IntegralAverage extends UnboundFunction {
+  override def name(): String = "iavg"
+
+  override def bind(inputType: StructType): BoundFunction = {
+    if (inputType.fields.length > 1) {
+      throw new UnsupportedOperationException("Too many arguments")
+    }
+
+    inputType.fields(0).dataType match {
+      case _: IntegerType => IntAverage
+      case _: LongType => LongAverage
+      case dataType =>
+        throw new UnsupportedOperationException(s"Unsupported non-integral type: $dataType")
+    }
+  }
+
+  override def description(): String =
+    """iavg: produces an average using integer division, ignoring nulls
+      |  iavg(int) -> int
+      |  iavg(bigint) -> bigint""".stripMargin
+}
+
+case class StrLen(impl: BoundFunction) extends UnboundFunction {
+  override def name(): String = "strlen"
+
+  override def bind(inputType: StructType): BoundFunction = {
+    if (inputType.fields.length != 1) {
+      throw new UnsupportedOperationException("Expect exactly one argument");
+    }
+    inputType.fields(0).dataType match {
+      case StringType => impl
+      case _ =>
+        throw new UnsupportedOperationException("Expect StringType")
+    }
+  }
+
+  override def description(): String =
+    "strlen: returns the length of the input string  strlen(string) -> int"
+}
+
 class DataSourceV2FunctionSuite extends DatasourceV2SQLBase {
-  private val emptyProps: util.Map[String, String] = Collections.emptyMap[String, String]
+  private val emptyProps: java.util.Map[String, String] = Collections.emptyMap[String, String]
 
   private def addFunction(ident: Identifier, fn: UnboundFunction): Unit = {
     catalog("testcat").asInstanceOf[InMemoryCatalog].createFunction(ident, fn)
   }
 
   test("undefined function") {
-    assert(intercept[AnalysisException](
-      sql("SELECT testcat.non_exist('abc')").collect()
-    ).getMessage.contains("Undefined function"))
+    checkError(
+      exception = intercept[AnalysisException](
+        sql("SELECT testcat.non_exist('abc')").collect()
+      ),
+      errorClass = "UNRESOLVED_ROUTINE",
+      parameters = Map(
+        "routineName" -> "`testcat`.`non_exist`",
+        "searchPath" -> "[`system`.`builtin`, `system`.`session`, `testcat`.`default`]"),
+      context = ExpectedContext(
+        fragment = "testcat.non_exist('abc')",
+        start = 7,
+        stop = 30))
   }
 
   test("non-function catalog") {
     withSQLConf("spark.sql.catalog.testcat" -> classOf[BasicInMemoryTableCatalog].getName) {
-      assert(intercept[AnalysisException](
-        sql("SELECT testcat.strlen('abc')").collect()
-      ).getMessage.contains("is not a FunctionCatalog"))
+      checkError(
+        exception = intercept[AnalysisException](
+          sql("SELECT testcat.strlen('abc')").collect()
+        ),
+        errorClass = "_LEGACY_ERROR_TEMP_1184",
+        parameters = Map("plugin" -> "testcat", "ability" -> "functions")
+      )
     }
   }
 
+  test("DESCRIBE FUNCTION: only support session catalog") {
+    addFunction(Identifier.of(Array.empty, "abc"), new JavaStrLen(new JavaStrLenNoImpl))
+
+    checkError(
+      exception = intercept[AnalysisException] {
+        sql("DESCRIBE FUNCTION testcat.abc")
+      },
+      errorClass = "_LEGACY_ERROR_TEMP_1184",
+      parameters = Map(
+        "plugin" -> "testcat",
+        "ability" -> "functions"
+      )
+    )
+
+    checkError(
+      exception = intercept[AnalysisException] {
+        sql("DESCRIBE FUNCTION default.ns1.ns2.fun")
+      },
+      errorClass = "REQUIRES_SINGLE_PART_NAMESPACE",
+      parameters = Map(
+        "sessionCatalog" -> "spark_catalog",
+        "namespace" -> "`default`.`ns1`.`ns2`")
+    )
+  }
+
+  test("DROP FUNCTION: only support session catalog") {
+    addFunction(Identifier.of(Array.empty, "abc"), new JavaStrLen(new JavaStrLenNoImpl))
+
+    val e = intercept[AnalysisException] {
+      sql("DROP FUNCTION testcat.abc")
+    }
+    assert(e.message.contains("Catalog testcat does not support DROP FUNCTION"))
+
+    val e1 = intercept[AnalysisException] {
+      sql("DROP FUNCTION default.ns1.ns2.fun")
+    }
+    assert(e1.message.contains("requires a single-part namespace"))
+  }
+
+  test("CREATE FUNCTION: only support session catalog") {
+    val e = intercept[AnalysisException] {
+      sql("CREATE FUNCTION testcat.ns1.ns2.fun as 'f'")
+    }
+    assert(e.message.contains("Catalog testcat does not support CREATE FUNCTION"))
+
+    val e1 = intercept[AnalysisException] {
+      sql("CREATE FUNCTION default.ns1.ns2.fun as 'f'")
+    }
+    assert(e1.message.contains("requires a single-part namespace"))
+  }
+
+  test("REFRESH FUNCTION: only support session catalog") {
+    addFunction(Identifier.of(Array.empty, "abc"), new JavaStrLen(new JavaStrLenNoImpl))
+
+    val e = intercept[AnalysisException] {
+      sql("REFRESH FUNCTION testcat.abc")
+    }
+    assert(e.message.contains("Catalog testcat does not support REFRESH FUNCTION"))
+
+    val e1 = intercept[AnalysisException] {
+      sql("REFRESH FUNCTION default.ns1.ns2.fun")
+    }
+    assert(e1.message.contains("requires a single-part namespace"))
+  }
+
   test("built-in with non-function catalog should still work") {
     withSQLConf(SQLConf.DEFAULT_CATALOG.key -> "testcat",
       "spark.sql.catalog.testcat" -> classOf[BasicInMemoryTableCatalog].getName) {
@@ -147,8 +321,7 @@ class DataSourceV2FunctionSuite extends DatasourceV2SQLBase {
   test("scalar function: bad magic method") {
     catalog("testcat").asInstanceOf[SupportsNamespaces].createNamespace(Array("ns"), emptyProps)
     addFunction(Identifier.of(Array("ns"), "strlen"), StrLen(StrLenBadMagic))
-    assert(intercept[SparkException](sql("SELECT testcat.ns.strlen('abc')").collect())
-      .getMessage.contains("Cannot find a compatible"))
+    intercept[UnsupportedOperationException](sql("SELECT testcat.ns.strlen('abc')").collect())
   }
 
   test("scalar function: bad magic method with default impl") {
@@ -160,17 +333,42 @@ class DataSourceV2FunctionSuite extends DatasourceV2SQLBase {
   test("scalar function: no implementation found") {
     catalog("testcat").asInstanceOf[SupportsNamespaces].createNamespace(Array("ns"), emptyProps)
     addFunction(Identifier.of(Array("ns"), "strlen"), StrLen(StrLenNoImpl))
-    intercept[SparkException](sql("SELECT testcat.ns.strlen('abc')").collect())
+    intercept[UnsupportedOperationException](sql("SELECT testcat.ns.strlen('abc')").collect())
   }
 
   test("scalar function: invalid parameter type or length") {
     catalog("testcat").asInstanceOf[SupportsNamespaces].createNamespace(Array("ns"), emptyProps)
     addFunction(Identifier.of(Array("ns"), "strlen"), StrLen(StrLenDefault))
 
-    assert(intercept[AnalysisException](sql("SELECT testcat.ns.strlen(42)"))
-      .getMessage.contains("Expect StringType"))
-    assert(intercept[AnalysisException](sql("SELECT testcat.ns.strlen('a', 'b')"))
-      .getMessage.contains("Expect exactly one argument"))
+    checkError(
+      exception = intercept[AnalysisException](sql("SELECT testcat.ns.strlen(42)")),
+      errorClass = "_LEGACY_ERROR_TEMP_1198",
+      parameters = Map(
+        "unbound" -> "strlen",
+        "arguments" -> "int",
+        "unsupported" -> "Expect StringType"
+      ),
+      context = ExpectedContext(
+        fragment = "testcat.ns.strlen(42)",
+        start = 7,
+        stop = 27
+      )
+    )
+
+    checkError(
+      exception = intercept[AnalysisException](sql("SELECT testcat.ns.strlen('a', 'b')")),
+      errorClass = "_LEGACY_ERROR_TEMP_1198",
+      parameters = Map(
+        "unbound" -> "strlen",
+        "arguments" -> "string, string",
+        "unsupported" -> "Expect exactly one argument"
+      ),
+      context = ExpectedContext(
+        fragment = "testcat.ns.strlen('a', 'b')",
+        start = 7,
+        stop = 33
+      )
+    )
   }
 
   test("scalar function: default produceResult in Java") {
@@ -213,22 +411,52 @@ class DataSourceV2FunctionSuite extends DatasourceV2SQLBase {
     catalog("testcat").asInstanceOf[SupportsNamespaces].createNamespace(Array("ns"), emptyProps)
     addFunction(Identifier.of(Array("ns"), "strlen"),
       new JavaStrLen(new JavaStrLenNoImpl))
-    assert(intercept[AnalysisException](sql("SELECT testcat.ns.strlen('abc')").collect())
-      .getMessage.contains("neither implement magic method nor override 'produceResult'"))
+    // TODO assign a error-classes name
+    checkError(
+      exception = intercept[AnalysisException](sql("SELECT testcat.ns.strlen('abc')").collect()),
+      errorClass = null,
+      parameters = Map.empty,
+      context = ExpectedContext(
+        fragment = "testcat.ns.strlen('abc')",
+        start = 7,
+        stop = 30
+      )
+    )
   }
 
   test("SPARK-35390: scalar function w/ bad input types") {
     catalog("testcat").asInstanceOf[SupportsNamespaces].createNamespace(Array("ns"), emptyProps)
     addFunction(Identifier.of(Array("ns"), "strlen"), StrLen(StrLenBadInputTypes))
-    assert(intercept[AnalysisException](sql("SELECT testcat.ns.strlen('abc')").collect())
-        .getMessage.contains("parameters returned from 'inputTypes()'"))
+    checkError(
+      exception = intercept[AnalysisException](sql("SELECT testcat.ns.strlen('abc')").collect()),
+      errorClass = "_LEGACY_ERROR_TEMP_1199",
+      parameters = Map(
+        "bound" -> "strlen_bad_input_types",
+        "argsLen" -> "1",
+        "inputTypesLen" -> "2"
+      ),
+      context = ExpectedContext(
+        fragment = "testcat.ns.strlen('abc')",
+        start = 7,
+        stop = 30
+      )
+    )
   }
 
   test("SPARK-35390: scalar function w/ mismatch type parameters from magic method") {
     catalog("testcat").asInstanceOf[SupportsNamespaces].createNamespace(Array("ns"), emptyProps)
     addFunction(Identifier.of(Array("ns"), "add"), new JavaLongAdd(new JavaLongAddMismatchMagic))
-    assert(intercept[AnalysisException](sql("SELECT testcat.ns.add(1L, 2L)").collect())
-        .getMessage.contains("neither implement magic method nor override 'produceResult'"))
+    // TODO assign a error-classes name
+    checkError(
+      exception = intercept[AnalysisException](sql("SELECT testcat.ns.add(1L, 2L)").collect()),
+      errorClass = null,
+      parameters = Map.empty,
+      context = ExpectedContext(
+        fragment = "testcat.ns.add(1L, 2L)",
+        start = 7,
+        stop = 28
+      )
+    )
   }
 
   test("SPARK-35390: scalar function w/ type coercion") {
@@ -242,10 +470,33 @@ class DataSourceV2FunctionSuite extends DatasourceV2SQLBase {
       checkAnswer(sql(s"SELECT testcat.ns.$name(42L, 58)"), Row(100) :: Nil)
       checkAnswer(sql(s"SELECT testcat.ns.$name(42, 58L)"), Row(100) :: Nil)
 
+      val paramIndex = name match {
+        case "add" => "1"
+        case "add2" => "2"
+        case "add3" => "1"
+      }
+
       // can't cast date time interval to long
-      assert(intercept[AnalysisException](
-        sql(s"SELECT testcat.ns.$name(date '2021-06-01' - date '2011-06-01', 93)").collect())
-          .getMessage.contains("due to data type mismatch"))
+      val sqlText = s"SELECT testcat.ns.$name(date '2021-06-01' - date '2011-06-01', 93)"
+      checkErrorMatchPVals(
+        exception = intercept[AnalysisException] {
+          sql(sqlText).collect()
+        },
+        errorClass = "DATATYPE_MISMATCH.UNEXPECTED_INPUT_TYPE",
+        sqlState = None,
+        parameters = Map(
+          "sqlExpr" -> ".*",
+          "paramIndex" -> paramIndex,
+          "inputSql" -> "\"\\(DATE '2021-06-01' - DATE '2011-06-01'\\)\"",
+          "inputType" -> "\"INTERVAL DAY\"",
+          "requiredType" -> "\"BIGINT\""
+        ),
+        context = ExpectedContext(
+          fragment = s"testcat.ns.$name(date '2021-06-01' - date '2011-06-01', 93)",
+          start = 7,
+          stop = sqlText.length - 1
+        )
+      )
     }
   }
 
@@ -286,8 +537,17 @@ class DataSourceV2FunctionSuite extends DatasourceV2SQLBase {
     catalog("testcat").asInstanceOf[SupportsNamespaces].createNamespace(Array("ns"), emptyProps)
     addFunction(Identifier.of(Array("ns"), "strlen"), StrLen(BadBoundFunction))
 
-    assert(intercept[AnalysisException](sql("SELECT testcat.ns.strlen('abc')"))
-      .getMessage.contains("does not implement ScalarFunction or AggregateFunction"))
+    checkError(
+      exception = intercept[AnalysisException](
+        sql("SELECT testcat.ns.strlen('abc')")),
+      errorClass = "INVALID_UDF_IMPLEMENTATION",
+      parameters = Map(
+        "funcName" -> "`bad_bound_func`"),
+      context = ExpectedContext(
+        fragment = "testcat.ns.strlen('abc')",
+        start = 7,
+        stop = 30)
+    )
   }
 
   test("aggregate function: lookup int average") {
@@ -341,8 +601,20 @@ class DataSourceV2FunctionSuite extends DatasourceV2SQLBase {
       addFunction(Identifier.of(Array("ns"), "avg"), IntegralAverage)
 
       Seq(1.toShort, 2.toShort).toDF("i").write.saveAsTable(t)
-      assert(intercept[AnalysisException](sql(s"SELECT testcat.ns.avg(i) from $t"))
-        .getMessage.contains("Unsupported non-integral type: ShortType"))
+      checkError(
+        exception = intercept[AnalysisException](sql(s"SELECT testcat.ns.avg(i) from $t")),
+        errorClass = "_LEGACY_ERROR_TEMP_1198",
+        parameters = Map(
+          "unbound" -> "iavg",
+          "arguments" -> "smallint",
+          "unsupported" -> "Unsupported non-integral type: ShortType"
+        ),
+        context = ExpectedContext(
+          fragment = "testcat.ns.avg(i)",
+          start = 7,
+          stop = 23
+        )
+      )
     }
   }
 
@@ -361,9 +633,25 @@ class DataSourceV2FunctionSuite extends DatasourceV2SQLBase {
         Row(BigDecimal(50.5)) :: Nil)
 
       // can't cast interval to decimal
-      assert(intercept[AnalysisException](sql("SELECT testcat.ns.avg(*) from values" +
-          " (date '2021-06-01' - date '2011-06-01'), (date '2000-01-01' - date '1900-01-01')"))
-          .getMessage.contains("due to data type mismatch"))
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql("SELECT testcat.ns.avg(*) from values " +
+            "(date '2021-06-01' - date '2011-06-01'), (date '2000-01-01' - date '1900-01-01')")
+        },
+        errorClass = "DATATYPE_MISMATCH.UNEXPECTED_INPUT_TYPE",
+        parameters = Map(
+          "sqlExpr" -> "\"v2aggregator(col1)\"",
+          "paramIndex" -> "1",
+          "inputSql" -> "\"col1\"",
+          "inputType" -> "\"INTERVAL DAY\"",
+          "requiredType" -> "\"DECIMAL(38,18)\""
+        ),
+        context = ExpectedContext(
+          fragment = "testcat.ns.avg(*)",
+          start = 7,
+          stop = 23
+        )
+      )
     }
   }
 
@@ -392,22 +680,14 @@ class DataSourceV2FunctionSuite extends DatasourceV2SQLBase {
     }
   }
 
-  private case class StrLen(impl: BoundFunction) extends UnboundFunction {
-    override def description(): String =
-      """strlen: returns the length of the input string
-        |  strlen(string) -> int""".stripMargin
-    override def name(): String = "strlen"
+  test("SPARK-44930: Fold deterministic ApplyFunctionExpression") {
+    catalog("testcat").asInstanceOf[SupportsNamespaces].createNamespace(Array("ns"), emptyProps)
+    addFunction(Identifier.of(Array("ns"), "strlen"), StrLen(StrLenDefault))
 
-    override def bind(inputType: StructType): BoundFunction = {
-      if (inputType.fields.length != 1) {
-        throw new UnsupportedOperationException("Expect exactly one argument");
-      }
-      inputType.fields(0).dataType match {
-        case StringType => impl
-        case _ =>
-          throw new UnsupportedOperationException("Expect StringType")
-      }
-    }
+    val df1 = sql("SELECT testcat.ns.strlen('abc') as col1")
+    val df2 = sql("SELECT 3 as col1")
+    comparePlans(df1.queryExecution.optimizedPlan, df2.queryExecution.optimizedPlan)
+    checkAnswer(df1, Row(3) :: Nil)
   }
 
   private case object StrLenDefault extends ScalarFunction[Int] {
@@ -475,84 +755,6 @@ class DataSourceV2FunctionSuite extends DatasourceV2SQLBase {
     override def name(): String = "bad_bound_func"
   }
 
-  object IntegralAverage extends UnboundFunction {
-    override def name(): String = "iavg"
-
-    override def bind(inputType: StructType): BoundFunction = {
-      if (inputType.fields.length > 1) {
-        throw new UnsupportedOperationException("Too many arguments")
-      }
-
-      inputType.fields(0).dataType match {
-        case _: IntegerType => IntAverage
-        case _: LongType => LongAverage
-        case dataType =>
-          throw new UnsupportedOperationException(s"Unsupported non-integral type: $dataType")
-      }
-    }
-
-    override def description(): String =
-      """iavg: produces an average using integer division, ignoring nulls
-        |  iavg(int) -> int
-        |  iavg(bigint) -> bigint""".stripMargin
-  }
-
-  object IntAverage extends AggregateFunction[(Int, Int), Int] {
-    override def name(): String = "iavg"
-    override def inputTypes(): Array[DataType] = Array(IntegerType)
-    override def resultType(): DataType = IntegerType
-
-    override def newAggregationState(): (Int, Int) = (0, 0)
-
-    override def update(state: (Int, Int), input: InternalRow): (Int, Int) = {
-      if (input.isNullAt(0)) {
-        state
-      } else {
-        val i = input.getInt(0)
-        state match {
-          case (_, 0) =>
-            (i, 1)
-          case (total, count) =>
-            (total + i, count + 1)
-        }
-      }
-    }
-
-    override def merge(leftState: (Int, Int), rightState: (Int, Int)): (Int, Int) = {
-      (leftState._1 + rightState._1, leftState._2 + rightState._2)
-    }
-
-    override def produceResult(state: (Int, Int)): Int = state._1 / state._2
-  }
-
-  object LongAverage extends AggregateFunction[(Long, Long), Long] {
-    override def name(): String = "iavg"
-    override def inputTypes(): Array[DataType] = Array(LongType)
-    override def resultType(): DataType = LongType
-
-    override def newAggregationState(): (Long, Long) = (0L, 0L)
-
-    override def update(state: (Long, Long), input: InternalRow): (Long, Long) = {
-      if (input.isNullAt(0)) {
-        state
-      } else {
-        val l = input.getLong(0)
-        state match {
-          case (_, 0L) =>
-            (l, 1)
-          case (total, count) =>
-            (total + l, count + 1L)
-        }
-      }
-    }
-
-    override def merge(leftState: (Long, Long), rightState: (Long, Long)): (Long, Long) = {
-      (leftState._1 + rightState._1, leftState._2 + rightState._2)
-    }
-
-    override def produceResult(state: (Long, Long)): Long = state._1 / state._2
-  }
-
   object UnboundDecimalAverage extends UnboundFunction {
     override def name(): String = "decimal_avg"
 
