@@ -200,19 +200,4 @@ class HiveExternalCatalogSuite extends ExternalCatalogSuite {
       assert(alteredTable.provider === Some("foo"))
     })
   }
-
-  test("SPARK-39203: Rewrite table location to absolute location based on database location") {
-    val tableLocation1 = CatalogUtils.stringToURI("/user/hive/warehouse/t1")
-    val tableLocation2 = CatalogUtils.stringToURI("viewfs://clusterB/user/hive/warehouse/t2")
-    val dbLocation = CatalogUtils.stringToURI("viewfs://clusterA/user/hive/warehouse/")
-
-    assert(HiveExternalCatalog.toAbsoluteURI(tableLocation1, Some(dbLocation))
-      .equals(CatalogUtils.stringToURI("viewfs://clusterA/user/hive/warehouse/t1")))
-
-    assert(HiveExternalCatalog.toAbsoluteURI(tableLocation1, None)
-      .equals(tableLocation1))
-
-    assert(HiveExternalCatalog.toAbsoluteURI(tableLocation2, Some(dbLocation))
-      .equals(tableLocation2))
-  }
 }
