@@ -113,36 +113,66 @@ class HiveOrcSourceSuite extends OrcSuite with TestHiveSingleton {
       val orcDir = new File(dir, "orc").getCanonicalPath
 
       // write path
-      var msg = intercept[AnalysisException] {
-        sql("select interval 1 days").write.mode("overwrite").orc(orcDir)
-      }.getMessage
-      assert(msg.contains("Cannot save interval data type into external storage."))
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql("select interval 1 days").write.mode("overwrite").orc(orcDir)
+        },
+        errorClass = "UNSUPPORTED_DATA_TYPE_FOR_DATASOURCE",
+        parameters = Map(
+          "columnName" -> "`INTERVAL '1' DAY`",
+          "columnType" -> "\"INTERVAL DAY\"",
+          "format" -> "ORC")
+      )
 
-      msg = intercept[AnalysisException] {
-        sql("select null").write.mode("overwrite").orc(orcDir)
-      }.getMessage
-      assert(msg.contains("ORC data source does not support null data type."))
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql("select null").write.mode("overwrite").orc(orcDir)
+        },
+        errorClass = "UNSUPPORTED_DATA_TYPE_FOR_DATASOURCE",
+        parameters = Map(
+          "columnName" -> "`NULL`",
+          "columnType" -> "\"VOID\"",
+          "format" -> "ORC")
+      )
 
-      msg = intercept[AnalysisException] {
-        spark.udf.register("testType", () => new IntervalData())
-        sql("select testType()").write.mode("overwrite").orc(orcDir)
-      }.getMessage
-      assert(msg.contains("ORC data source does not support interval data type."))
+      checkError(
+        exception = intercept[AnalysisException] {
+          spark.udf.register("testType", () => new IntervalData())
+          sql("select testType()").write.mode("overwrite").orc(orcDir)
+        },
+        errorClass = "UNSUPPORTED_DATA_TYPE_FOR_DATASOURCE",
+        parameters = Map(
+          "columnName" -> "`testType()`",
+          "columnType" -> "\"INTERVAL\"",
+          "format" -> "ORC")
+      )
 
       // read path
-      msg = intercept[AnalysisException] {
-        val schema = StructType(StructField("a", CalendarIntervalType, true) :: Nil)
-        spark.range(1).write.mode("overwrite").orc(orcDir)
-        spark.read.schema(schema).orc(orcDir).collect()
-      }.getMessage
-      assert(msg.contains("ORC data source does not support interval data type."))
+      checkError(
+        exception = intercept[AnalysisException] {
+          val schema = StructType(StructField("a", CalendarIntervalType, true) :: Nil)
+          spark.range(1).write.mode("overwrite").orc(orcDir)
+          spark.read.schema(schema).orc(orcDir).collect()
+        },
+        errorClass = "UNSUPPORTED_DATA_TYPE_FOR_DATASOURCE",
+        parameters = Map(
+          "columnName" -> "`a`",
+          "columnType" -> "\"INTERVAL\"",
+          "format" -> "ORC")
+      )
 
-      msg = intercept[AnalysisException] {
-        val schema = StructType(StructField("a", new IntervalUDT(), true) :: Nil)
-        spark.range(1).write.mode("overwrite").orc(orcDir)
-        spark.read.schema(schema).orc(orcDir).collect()
-      }.getMessage
-      assert(msg.contains("ORC data source does not support interval data type."))
+      checkError(
+        exception = intercept[AnalysisException] {
+          val schema = StructType(StructField("a", new IntervalUDT(), true) :: Nil)
+          spark.range(1).write.mode("overwrite").orc(orcDir)
+          spark.read.schema(schema).orc(orcDir).collect()
+        },
+        errorClass = "UNSUPPORTED_DATA_TYPE_FOR_DATASOURCE",
+        parameters = Map(
+          "columnName" -> "`a`",
+          "columnType" -> "\"INTERVAL\"",
+          "format" -> "ORC")
+      )
     }
   }
 
