@@ -28,6 +28,7 @@ import scala.concurrent._
 import scala.concurrent.duration._
 import scala.language.implicitConversions
 
+import org.apache.commons.lang3.{JavaVersion, SystemUtils}
 import org.apache.hadoop.conf.Configuration
 import org.apache.hadoop.fs.Path
 import org.mockito.ArgumentCaptor
@@ -476,7 +477,13 @@ class BatchedWriteAheadLogSuite extends CommonWriteAheadLogTests(
     val batchedWal = new BatchedWriteAheadLog(wal, sparkConf)
 
     val e = intercept[SparkException] {
-      val buffer = mock[ByteBuffer]
+      val buffer = if (SystemUtils.isJavaVersionAtMost(JavaVersion.JAVA_17)) {
+        mock[ByteBuffer]
+      } else {
+        // SPARK-40731: Use a 0 size `ByteBuffer` instead of `mock[ByteBuffer]`
+        // for Java 17+ due to mockito 4 can't mock/spy sealed class
+        ByteBuffer.allocate(0)
+      }
       batchedWal.write(buffer, 2L)
     }
     assert(e.getCause.getMessage === "Hello!")
@@ -546,7 +553,14 @@ class BatchedWriteAheadLogSuite extends CommonWriteAheadLogTests(
     batchedWal.close()
     verify(wal, times(1)).close()
 
-    intercept[IllegalStateException](batchedWal.write(mock[ByteBuffer], 12L))
+    val buffer = if (SystemUtils.isJavaVersionAtMost(JavaVersion.JAVA_17)) {
+      mock[ByteBuffer]
+    } else {
+      // SPARK-40731: Use a 0 size `ByteBuffer` instead of `mock[ByteBuffer]`
+      // for Java 17+ due to mockito 4 can't mock/spy sealed class
+      ByteBuffer.allocate(0)
+    }
+    intercept[IllegalStateException](batchedWal.write(buffer, 12L))
   }
 
   test("BatchedWriteAheadLog - fail everything in queue during shutdown") {
